<?php
/*
  サンプルプログラム(index.php) [PHP >= 5.3]
  Copyright (C) 2012 Masanori OKIBAYASHI all rights reserved.
*/


/******************************

       Webブラウザから
       アクセスされるWeb API

 ******************************/

  // ライブラリの読み込み
  require 'mf.php';           // Web APIプログラミング用
  require 'palsline.php';     // サービスのメイン(PalsLineクラス)


  // レスポンスのほとんどはJSON形式
  function response($obj) {
    header('Content-Type: application/json');
    print json_encode($obj);
    exit;
  }

  // エラーがある場合のステータスコード
  function bad_request() {
    header('HTTP/1.1 400 Bad Request');
    exit;
  }

  // ログアウトまでの時刻を延長(現在時刻の30分後まで)
  // セッションIDごとに設定
  function extendSession($session_id, $app) {
    $time = time() + 60 * 30;
    $result = $app->extendSession($session_id, $time);
    if ($result) {
      setCookie('session_id', $session_id, $time, '/');
      return true;
    } else {
      setCookie('session_id', '', time() - 3600, '/');
      bad_request();
    }
  }


  /**********************

          Web API

  **********************/

  // サービスのメインとなるアプリケーション
  $app = new PalsLine();

  // ログイン(POST /login)
  post('/login', function() use ($app) {
    $result = $app->login($_POST['user'], $_POST['pass']);
    extendSession($result['session_id'], $app);
    response(array('login' => $result));
  });

  // ログアウト(GET /logout)
  get('/logout', function($params) use ($app) {
    $result = $app->logout($_COOKIE['session_id']);
    setCookie('session_id', '', time() - 3600, '/');
    response(array('logout' => $result));
  });

  // メッセージ一覧の参照(GET /messages/:rows/:offset)
  get('/messages/:rows/:offset', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->getMessages($_COOKIE['session_id'],
                                $params['offset'], $params['rows']);
    response(array('messages' => $result));
  });

  // メッセージ削除(DELETE /message/:id)
  delete('/message/:id', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->deleteMessage($_COOKIE['session_id'], $params['id']);
    response(array('deleteMessage' => $result));
  });

  // 写真一覧の参照(GET /photos/:rows/:offset)
  get('/photos/:rows/:offset', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->getPhotos($_COOKIE['session_id'],
                              $params['offset'], $params['rows']);
    response(array('photos' => $result));
  });

  // 写真データ参照表示(GET /image/:id)
  // レスポンスは写真データ(URLを<img>のsrc属性に設定)
  get('/image/:id', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->getImage($_COOKIE['session_id'], $params['id']);
    header('Content-Type: ' . $result['type']);
    echo $result['data'];
    exit;
  });

  // 写真削除(DELETE /photo/:id)
  delete('/photo/:id', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->deletePhoto($_COOKIE['session_id'], $params['id']);
    response(array('deletePhoto' => $result));
  });

  // ユーザー一覧(GET /users)
  // メッセージ共有ユーザー設定用
  get('/users', function() use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $results = $app->getUsers($_COOKIE['session_id']);
    response(array('users' => $results));
  });

  // メッセージと写真の投稿(POST /upload)
  post('/upload', function() use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->addMessage($_COOKIE['session_id'],
                               $_POST['message'], $_POST['share'],
                               $_FILES['file']);
    response(array('message_id' => $result));
  });

  // メッセージ参照(GET /message/:id)
  get('/message/:id', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->getMessage($_COOKIE['session_id'], $params['id']);
    response(array('message' => $result));
  });

  // メッセージ共有ユーザー参照(GET /shared/:message_id)
  // 共有ユーザー設定用
  get('/shared/:message_id', function($params) use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $results = $app->getShared($_COOKIE['session_id'], $params['message_id']);
    response(array('shared' => $results));
  });

  // メッセージ共有ユーザー設定(POST /share)
  post('/share', function() use ($app) {
    extendSession($_COOKIE['session_id'], $app);
    $result = $app->addShare(
      $_COOKIE['session_id'], $_POST['message_id'], $_POST['share']
    );
    response(array('addShare' => $result));
  });
