<?php
/*
  サンプルプログラム用マイクロフレームワーク(mf.php) [PHP >= 5.3]
  Copyright (C) 2012 Masanori OKIBAYASHI all rights reserved.
*/


/******************************

       処理を振り分ける
       フレームワーク

 ******************************/

  class __MicroFramework {

    private $routes;  // URIによる処理の振り分け

    // 処理の振り分け設定を追加(メソッド名をHTTPメソッドとみなす)
    public function __call($name, $args) {
      $this->add_route(strtoupper($name), $args);
    }

    // 処理の振り分け設定を追加
    private function add_route($method, $args /* query, func */) {
      preg_match_all('/:([^\\\&@\/\?\:\=]+)/',
                     $_SERVER['SCRIPT_NAME'] . $args[0],
                     $names);
      $expr = preg_replace('/([\/\?\=])/', '\\\$1', $args[0]);
      $expr = str_replace('/*/', '/.+?/', $expr);
      $expr = '/' . preg_replace('/:[^\\\&@\/\?\:\=]+/',
                                 '([^\\\&@\/\?\:\=]+)', $expr) . '/';
      array_push($this->routes, array($method, $expr, $names[1], $args[1]));
    }

    // HTTPメソッドとURIに基づいて実行する処理を振り分け
    private function run() {
      foreach($this->routes as $route /* method, expr, names, func */) {
        if ($route[0] === $_SERVER['REQUEST_METHOD'] &&
            preg_match($route[1], $_SERVER['REQUEST_URI'], $values)) {
          array_shift($values);
          $values = array_map(function($value) { return urldecode($value); }, $values);
          @$route[3](array_combine($route[2], $values));
          break;
        }
      }
    }

    // コンストラクタ(初期設定)
    public function __construct() {
      $this->routes = array();
    }

    // デストラクタ(振り分けられた処理を実行)
    public function __destruct() {
      $this->run();
    }
  }



/******************************

       短縮メソッドの定義

 ******************************/

  $__mf = new __MicroFramework();

  // GETメソッド用
  function get($query, $func) {
    global $__mf;
    $__mf->get($query, $func);
  }

  // POSTメソッド用
  function post($query, $func) {
    global $__mf;
    $__mf->post($query, $func);
  }

  // DELETEメソッド用
  function delete($query, $func) {
    global $__mf;
    $__mf->delete($query, $func);
  }

  // PUTメソッド用
  function put($query, $func) {
    global $__mf;
    $__mf->put($query, $func);
  }

  // HEADメソッド用
  function head($query, $func) {
    global $__mf;
    $__mf->head($query, $func);
  }
