/*
  サンプルプログラム (JavaScript)
  Copyright 2012 (C) Masanori OKIBAYASHI all rights reserved.
*/

jQuery(document).on('mobileinit', function() {
  $.mobile.page.prototype.options.backBtnText   = '戻る';
  $.mobile.loader.prototype.options.textVisible = true;
  $.mobile.loader.prototype.options.text        = 'アクセス中...';
  $.mobile.pageLoadErrorMessage                 = 'アクセスできませんでした';
});

jQuery(window).one('pagecontainercreate', function() {
  $('[id^=tmpl-page-]').each(function() {
    $(this).tmpl().appendTo($.mobile.pageContainer);
  });
});

// ログイン画面の初期設定
jQuery(document).on('pageinit', '#page-login', function() {
  // フォームはsubmitイベントに対応させない
  $('#form-login').on('submit', false);

  // 「ログイン」ボタンがタップ/クリックされたら
  $('#btn-login').on('vclick', function() {
    // Web APIにアクセスしてログイン
    $.ajax({
      type: 'POST',
      url:  'index.php/login',
      data: $('#form-login').serialize()
    }).done(function(data, textStatus, jqXHR) {
      // ログインが成功したらフッタにユーザー名を表示
      if (data && data.login && data.login.user) {
        $('.user').text(data.login.user);
      } else {
        // ログインできなければダイアログを表示
        $.mobile.changePage('#dialog');
        return;
      }
      // パスワード入力フィールドを空欄にする
      $('#form-login :password').val('');
      // メッセージ一覧画面に遷移
      $.mobile.changePage('#page-messages');
    }).fail(function(jqXHR, textStatus, error) {
      // ログインできなければダイアログを表示
      $.mobile.changePage('#dialog');
    });
    return false;
  });

  // ログイン画面を表示するとき
  $(document).on('pagebeforeshow', '#page-login', function() {
    // Web APIにアクセスしてログアウト
    $.get('index.php/logout').always(function() {
      // 検索結果やユーザー名を削除
      $('.results, .user').empty();
    });
  });
});


// メッセージ一覧画面の初期設定
jQuery(document).on('pageinit', '#page-messages', function(ev) {
  // メッセージ一覧の表示
  // (rows: 画面に表示するメッセージの最大数, page: ページ数)
  function paginate(rows, page) {
    // Web APIへのアクセス
    $.ajax({
      type: 'GET',
      url:  'index.php/messages/' + rows + '/' + (page - 1) * rows
    }).done(function(data, textStatus, jqXHR) {
      if (!data || !data.messages) {
        // データを正常に受信できなければダイアログを表示
        $.mobile.changePage('#dialog');
        return;
      }
      // 一覧表示部分をクリア
      var results = $(ev.target).find('.results').empty();
      // 受信したメッセージ一覧を画面に表示
      $('#tmpl-messages').tmpl(data).appendTo(results).collapsible();
      // ページ送りボタンを画面に表示
      $('#tmpl-paginate').tmpl({page: page, rows: rows,
                                length: data.messages.length})
                         .appendTo(results)
                         .find('a').buttonMarkup({inline: true});
    }).fail(function(jqXHR, textStatus, error) {
      // データを正常に受信できなければダイアログを表示
      $.mobile.changePage('#dialog');
    });
  }

  // ページボタンがタップ/クリックされたら
  // data-page属性で設定されたページのメッセージ一覧を表示
  jQuery(ev.target).on('vclick', '.paginate a', function() {
    paginate(5, Number(this.getAttribute('data-page')));
  });

  // 画面が表示されるときの処理を設定
  jQuery(ev.target).on({
    pagebeforeshow: function() {
      // 画面を表示する前にメッセージ一覧を削除
      $('.results', this).empty();
    },
    pageshow: function() {
      // 画面を表示するときに最初のメッセージ一覧を表示
      paginate(5, 1);
    }
  });

  // 「more...」がタップ/クリックされたらメッセージ表示画面に遷移
  jQuery(ev.target).on('vclick', 'a.more', function() {
    $.mobile.changePage('#page-message',
                        {id: this.getAttribute('data-message')});
    return false;
  });

  // 「Delete」がタップ/クリックされたらそのメッセージを削除
  // (Web APIにアクセス)
  jQuery(ev.target).on('vclick', 'a.delete', function(e) {
    $.ajax({
      type: 'DELETE',
      url:  'index.php/message/' + e.target.getAttribute('data-message')
    }).done(function(data, textStatus, jqXHR) {
      paginate(5, 1);
    });
    return false;
  });
});


// 写真一覧画面の初期設定
jQuery(document).on('pageinit', '#page-photos', function(ev) {
  // 写真一覧の表示
  // (rows: 画面に表示するメッセージの最大数, page: ページ数)
  function paginate(rows, page) {
    // Web APIにアクセス
    $.ajax({
      type: 'GET',
      url:  'index.php/photos/' + rows + '/' + (page - 1) * rows
    }).done(function(data, textStatus, jqXHR) {
      if (!data || !data.photos) {
        // データを正常に受信できなければダイアログを表示
        $.mobile.changePage('#dialog');
        return;
      }
      // 一覧表示部分をクリア
      var results = $(ev.target).find('.results').empty();
      // 受信した写真一覧を画面に表示
      $('#tmpl-photos').tmpl(data).appendTo(results);
      // ページ送りボタンを画面に表示
      $('#tmpl-paginate').tmpl({page: page, rows: rows,
                                length: data.photos.length})
                         .appendTo(results)
                         .find('a').buttonMarkup({inline: true});
    }).fail(function(jqXHR, textStatus, error) {
      // データを正常に受信できなければダイアログを表示
      $.mobile.changePage('#dialog');
    });
  }

  // ページボタンがタップ/クリックされたら
  // data-page属性で設定されたページの写真一覧を表示
  jQuery(ev.target).on('vclick', '.paginate a', function() {
    paginate(6, Number(this.getAttribute('data-page')));
  });

  // 画面が表示されるときの処理を設定
  jQuery(ev.target).on({
    pagebeforeshow: function() {
      // 画面を表示する前に写真一覧を削除
      $('.results', this).empty();
    },
    pageshow: function() {
      // 画面を表示するときに最初の写真一覧を表示
      paginate(6, 1);
    }
  });

  // 画像がタップ/クリックされたらメッセージ表示画面に遷移
  jQuery(ev.target).on('vclick', 'a.more', function() {
    $.mobile.changePage('#page-message',
                        {id: this.getAttribute('data-message')});
    return false;
  });

  // 「Delete」がタップ/クリックされたらその写真を削除
  // (Web APIにアクセス)
  jQuery(ev.target).on('vclick', 'a.delete', function(e) {
    $.ajax({
      type: 'DELETE',
      url:  'index.php/photo/' + e.target.getAttribute('data-photo')
    }).done(function(data, textStatus, jqXHR) {
      paginate(6, 1);
    });
    return false;
  });
});


// メッセージ投稿画面の初期設定
jQuery(document).on('pageinit', '#page-upload', function(ev) {
  // フォーム
  var form = $('#form-upload', this);
  // 「写真」ボタンを構成する要素
  var btn  = $('input[type="button"]', form);

  // ファイル選択のフィールドは非表示
  form.find('input[type="file"]').hide();

  // 「写真」がタップ/クリックされたら
  // アップロードする写真を選択
  btn.on('vclick', function() {
    $('#form-upload :file').click();
  });

  // アップロードする写真の数を「写真」ボタンに表示
  $('input[type="file"]', form).on('change', function() {
    btn.val(
      '写真' +
      (this.files.length > 0 ? '(' + this.files.length + ')' : '')
    ).button('refresh');
    // 「投稿」ボタンにフォーカスを当てる
    $('input[type="submit"]', form).focus();
  });

  // メッセージの投稿
  $('input[type="submit"]', form).on('vclick', function() {
    var formData = new FormData(form[0]);
    // Web APIへのアクセス
    $.ajax({
      type: 'POST',
      url:  'index.php/upload',
      data: formData,
      processData: false,
      contentType: false
    }).done(function(data, textStatus, jqXHR) {
      if (!data || !data.message_id) {
        // 投稿できなければダイアログを表示
        $.mobile.changePage('#dialog');
        return;
      }
      // 投稿が完了したら
      // メッセージ一覧画面に遷移
      $.mobile.changePage('#page-messages');
    }).fail(function(jqXHR, textStatus, error) {
      // 投稿できなければダイアログを表示
      $.mobile.changePage('#dialog');
    }).always(function() {
      // 投稿フォームをリセット
      form[0].reset();
      // 「写真」ボタンに表示した写真の数も削除
      btn.val('写真').button('refresh');
    });
    return false;
  });
});


// メッセージ投稿画面表示時の処理を設定
jQuery(document).on('pagebeforeshow', '#page-upload', function(ev) {
  // ログインしているユーザー以外のユーザーのIDと名称を参照
  // (Web APIにアクセス)
  $.ajax({
    type: 'GET',
    url: 'index.php/users'
  }).done(function(data, textStatus, jqXHR) {
    if (!data || !data.users) {
      // 正常にデータを受信できなければダイアログを表示
      $.mobile.changePage('#dialog');
      return;
    }
    if (data.users.length + 0 !== data.users.length) {
      data.users = [data.users];
    }
    // メッセージを共有するユーザーを設定する
    // チェックボックスを画面に表示
    $('#form-upload .share').remove();
    $('#tmpl-share').tmpl({share: data.users}).appendTo('#form-upload')
                    .find('input[type="checkbox"]').checkboxradio()
                    .end().controlgroup();
  }).fail(function(jqXHR, textStatus, error) {
    // 正常にデータを受信できなければダイアログを表示
    $.mobile.changePage('#dialog');
  });
});


// メッセージ投稿画面が閉じられたときの処理を設定
jQuery(document).on('pagehide', '#page-upload', function(ev) {
  // メッセージ投稿フォーム
  var form = $('form', ev.target);
  // フォームをリセット
  form[0].reset();
  // 写真を選択した数も削除
  $('input[type="button"]', form).val('写真').button('refresh');
});


// メッセージ表示画面の初期設定
jQuery(document).on('pageinit', '#page-message', function(ev) {
  var content = $(':jqmData(role="content")', ev.target);
  // 左右のスワイプで表示するメッセージを切り替え
  content.on({
    swipeleft: function(e) {
      // 左のスワイプで次のメッセージを表示
      var nextPage = $('div.message', e.currentTarget)
                        .attr('data-next-message');
      if (!nextPage) return false;
      $.mobile.changePage('#page-message', { id: nextPage });
    },
    swiperight: function(e) {
      // 右のスワイプで前のメッセージを表示
      var prevPage = $('div.message', e.currentTarget)
                        .attr('data-prev-message');
      if (!prevPage) return false;
      $.mobile.changePage('#page-message', { id: prevPage });
    }
  });

  // 共有ユーザーを設定するためのフィールドを表示
  function share(form) {
    // メッセージを共有するユーザーを参照
    // (Web APIにアクセス)
    $.ajax({
      type: 'GET',
      url: 'index.php/shared/' + $('input[type="hidden"]', form).val(),
      dataFilter: function(response) {
        var data = $.parseJSON(response);
        if (!data || !data.shared) throw 'error';
        return response;
      }
    }).done(function(data, textStatus, jqXHR) {
      if (data.shared.length + 0 !== data.shared.length) {
        data.shared = [data.shared];
      }
      // レスポンスから入力フィールドを生成して<form>内に表示
      $('.share', form).remove();
      $('#tmpl-share').tmpl({share: data.shared}).prependTo(form)
                      .find('input[type="checkbox"]').checkboxradio()
                      .end().controlgroup();
    });
  }

  // 画面遷移の前に、画面に表示するメッセージのID
  // を取得してメッセージ表示画面を生成
  jQuery(document).on('pagebeforechange', function(evt, settings) {
    if (settings.toPage[0] !== ev.target) return;

    // 表示するページのID
    var id = settings.options.id;

    // IDの設定が無ければ以下を実行しない
    // (｢進む｣ボタンによる遷移の場合など)
    if (!id) return;

    // コンテンツ領域を削除
    content.empty();

    // 画面に表示するメッセージを参照
    // (Web APIにアクセス)
    $.ajax({
      type: 'GET',
      url:  'index.php/message/' + id,
      dataFilter: function(response) {
        // レスポンスにメッセージが存在するかを確認
        var data = $.parseJSON(response);
        if (!data || !data.message) throw 'error';
        return response;
      }
    }).done(function(data) {
      // メッセージ表示画面にメッセージを表示
      content.empty();
      var div = $('#tmpl-message').tmpl(data.message)
                                  .appendTo(content);
      // <form>が設定されたら実行
      $.when(div.promise()).done(function() {
        // メッセージ共有ユーザーを設定する
        // フォームに入力フィールドを生成
        var form = $('form', div);
        if (form.length < 1)  return;
        // 「追加/削除」ボタンがタップ/クリックされたら
        form.submit(function(e) {
          // メッセージを共有するユーザーを設定
          // (Web APIにアクセス)
          $.ajax({
            type: 'POST',
            url: 'index.php/share',
            data: $(e.target).serialize()
          }).done(function() {
            share(form);  // 入力フィールドを更新
          });
          return false;
        }).find('input[type="submit"]').button();
        share(form);  // 入力フィールドを表示
      });
    }).fail(function() {
      $.mobile.changePage('#dialog');
    });
  });
});
