CHARSET 'utf8';
SET CHARACTER SET 'utf8';

/*
  サンプルプログラム
  Copyright (C) 2012 Masanori OKIBAYASHI all rights reserved.
*/


/******************************

     データベース、テーブル、
     ビューの定義(MySQL)

 ******************************/

-- データベース作成(文字コードUTF-8)
DROP DATABASE IF EXISTS palsline;
CREATE DATABASE IF NOT EXISTS palsline DEFAULT CHARACTER SET utf8;

USE palsline;

--              --
-- テーブル定義 --
--              --

-- ユーザー
CREATE TABLE user (
  id     BIGINT      AUTO_INCREMENT PRIMARY KEY,  -- ユーザーID
  name   VARCHAR(50) NOT NULL UNIQUE,             -- ユーザー名(画面に表示)
  pass   VARCHAR(50) NOT NULL,                    -- パスワード(暗号化)
  regist TIMESTAMP   DEFAULT CURRENT_TIMESTAMP    -- 保存時刻
);

-- メッセージ
CREATE TABLE message (
  id      BIGINT    AUTO_INCREMENT PRIMARY KEY,   -- メッセージID
  user_id BIGINT    NOT NULL REFERENCES user(id), -- 投稿ユーザーID
  content TEXT      NOT NULL,                     -- メッセージ本文
  regist  TIMESTAMP DEFAULT CURRENT_TIMESTAMP     -- 保存時刻
);

-- メッセージと共に投稿する写真
CREATE TABLE photo (
  id         BIGINT       AUTO_INCREMENT PRIMARY KEY,       -- 写真ID
  message_id BIGINT       NOT NULL REFERENCES message(id),  -- メッセージID
  name       VARCHAR(100) NOT NULL,                         -- ファイル名
  type       VARCHAR(100) NOT NULL,                         -- MIMEタイプ
  data       MEDIUMBLOB   NOT NULL,                         -- 写真データ
  regist     TIMESTAMP    DEFAULT CURRENT_TIMESTAMP,        -- 保存時刻
  UNIQUE KEY (message_id, name)
);

-- メッセージを共有するユーザー
CREATE TABLE share (
  message_id BIGINT NOT NULL REFERENCES message(id),  -- メッセージID
  user_id    BIGINT NOT NULL REFERENCES user(id),     -- ユーザーID
  UNIQUE KEY (message_id, user_id)
);

-- セッション(ログイン/ログアウト)
CREATE TABLE session (
  id      BIGINT    AUTO_INCREMENT PRIMARY KEY,     -- セッションID
  user_id BIGINT    NOT NULL REFERENCES user(id),   -- ユーザーID
  login   TIMESTAMP DEFAULT  CURRENT_TIMESTAMP,     -- ログイン時刻
  logout  TIMESTAMP                                 -- ログアウト時刻
);

--            --
-- ビュー定義 --
--            --

-- メッセージと写真の対応
CREATE VIEW message_photo AS
  SELECT m.user_id, m.id AS message_id, m.content, m.regist,
         p.id AS photo_id, p.name, p.type, p.data
    FROM message m LEFT OUTER JOIN photo p
      ON p.message_id = m.id;

-- ログイン中のセッション
CREATE VIEW logging_session AS
  SELECT * FROM session WHERE NOW() >= login AND NOW() < logout;
